C     Last change:  HO    6 Oct 2005    9:29 pm
      Program Combine_CphLil
C
C                   ******************************
C                     C O M B I N E _ C P H L I L       (Sept 2005)
C                   ******************************
C
C
C     National Environmental Research Institute
C     Department of Atmospheric Environment
C     P.O. Box 358
C     DK-4000  Roskilde
C     DENMARK
C
C     The program was written by:
C      Helge Rordam Olesen
C
C      Phone:  +45 46 30 12 00 (ext. 1151)
C      Fax:    +45 46 30 12 14
C      E-mail: hro@dmu.dk
C
C
C
C  DESCRIPTION:
C  ------------
C     Utility for selecting data lines of interest in a file with
C     modelled Copenhagen or Lillestrom data, and combining them with observed
C     tracer concentration data.
C
C     The two files to be combined are
C     (1) modelled Copenhagen/Lillestrom data
C     (2) a master file with information on observed tracer concentrations.
C   
C     The outcome of the efforts are files that can be used as input for
C     the BOOT, RESIDUAL and SIGPLOT programmes (or for other purposes).
C
C
C  PROGRAM LOGIC:
C  ---------------
C     Various options (filenames and pathnames) are specified in the INI
C     file Combine_CphLil.ini, which must be present in the same folder as 
C     the executable.
C
C     First of all, this INI file is read.
C
C     For Copenhagen: The input files MODELLED.CPH (user-created) and 
C     MASTER.CPH (supplied) are opened.  
C     For Lillestrom, the input files are here called MODELLED.LIL and MASTER.LIL.

C     Records from the two files are read. 
c
C     Each time the data correspond in date, time and distance,a record is written
C     to four almost identical temporary files, BODY1, BODY2, BODYY1 and BODYY2
C
C     When the loops end, the BODY files contain essentially all information to
C     be output. However, headers must be included, and in the case of
C     the files of type .INP, also a 'footer' must be added.
C
C     Thus, four files with headers are created.
C
C     Finally, a DOS copy command is used in order to combine headers,
C     BODY and (in case of .INP) footer to the final output files.
C
C  DATA DICTIONARY:
C  -----------------
C     Mainly, usage of the variables is apparent from the program code.
C     Note, however:
C
C     Date and time variables from MODELLED.CPH (and .LIL) are designated by 
C     a name with an 'M';
C     MDY, MMO, MYR
C     MHR
C     JUL_M
C     D_TIME_M
C
C     while the corresponding variables from MASTER.CPH (and .LIL) are
C     designated by a name with an 'N':
C
C     NDY, NMO, NYR
C     NHR
C     JUL_N
C     D_TIME_N
C
C  REQUIRED FILES:
C  ----------------
C  Input:
C
C  Here in the explanation, names referring to Copenhagen (CPH) are used.
C  Copenhagen data and Lillestrom data are treated similarly,
C  so for Lillestrom, just use file names containing LIL instead of CPH.
C
C  Four input files are required:
C     Combine_CphLil.INI
C     MODELLED.CPH  (the file can have any name)
C     MASTER.CPH
C     CPH.BTT
C
C  Combine_CphLil.INI contains user specifications of filenames and path names.
C  It also specifies whether Copenhagen or Lillestrom data should be processed.
C

C  The primary input is a file with modelled concentrations,
C  MODELLED.CPH (or MODELLED.LIL), which should be prepared by the
C  user and contain modelled concentrations.
C  Although the name MODELLED.CPH is used here,the file can
C  have any name.
C
C  It must be placed in a folder specified in Combine_CphLil.ini.
C  Its name is also specified in Combine_CphLil.ini.
C  The format of MODELLED.CPH (or ...LIL) should be the following:
C
C  -  There is a one-line heading in MODELLED.CPH/MODELLED.LIL
C  -  There should be seven columns (or more), separated by blanks
C        (the values are read using free-format input).
C  -  There MUST be a line for all arc-hours (Copenhagen: 9 hours
C     times 13 distances, altogether 117 lines; Lillestrom:  8
C     periods times 14 distances, altogether 112 lines)
C  -  The distances must be those indicated in the samples below.
C
C  An example of the first fourteen lines of a MODELLED.CPH file is
C  shown below
C  [Disregard 'C' in the first column]
c
C   YR MO DY   HRE    DIST        ARCMAX/Q         CY/Q   
C   78  9 20   1417   1.900          332.          374.
C   78  9 20   1417   2.000          301.          355.
C   78  9 20   1417   2.100          275.          338.
C   78  9 20   1417   3.600          101.          196.
C   78  9 20   1417   3.700           97.          191.
C   78  9 20   1417   4.000           84.          178.
C   78  9 20   1417   4.100           81.          174.
C   78  9 20   1417   4.200           78.          170.
C   78  9 20   1417   5.300           55.          146.
C   78  9 20   1417   5.400           54.          145.
C   78  9 20   1417   5.900           49.          141.
C   78  9 20   1417   6.000           48.          140.
C   78  9 20   1417   6.100           47.          140.
                           ]


C  and an example of the first fifteen lines of a MODELLED.LIL file
C  is the following

C   YR MO DY   HRE    DIST        ARCMAX/Q         CY/Q   
C   87  1 10    945   0.140         6930.          313.
C   87  1 10    945   0.150         9439.          457.
C   87  1 10    945   0.160        12020.          620.
C   87  1 10    945   0.190        19044.         1167.
C   87  1 10    945   0.300        27819.         2691.
C   87  1 10    945   0.410        25066.         3314.
C   87  1 10    945   0.430        24351.         3376.
C   87  1 10    945   0.440        23968.         3400.
C   87  1 10    945   0.460        23167.         3436.
C   87  1 10    945   0.470        22755.         3449.
C   87  1 10    945   0.490        21924.         3464.
C   87  1 10    945   0.810        11659.         3045.
C   87  1 10    945   0.820        11448.         3027.
C   87  1 10    945   0.900         9941.         2885.
C
C  The secondary input is a file, MASTER.CPH (or MASTER.LIL), which
C  is provided together with this programme.  It has a one-line
C  heading, followed by 23 data lines (Copenhagen), or 22 data lines
C  (Lillestrom).
C
C     CP.BTT (and LIL.BTT) is provided together with this programme. It is
C     automatically copied into the bottom part of the output file of type .INP.
C
C     These two files must be placed in a folder specified in Combine_ChpLil.ini .
C
C     Below follows a list of input files and temporary files used
C     during program execution:
C
C     File name     Unit
C     MODELLED.CPH   10      <- input prepared by user (the name can be 
C                                specified in Combine_CphLil.ini).
C     MASTER.CPH     11      <- supplied with this code
C     BODY1          12      <- temporary file for body of INP file
C     BODY2          16      <- temporary file for body of scatter file
C     CPH.BTT        13      <- supplied with this code
C                               (bottom part of .INP files).
C     IN_HDR.INP     14      <- temporary file, header for .INP files
C     IN_HDR.DAT     15      <- temporary file, header for .DAT files
C     BODYY1         22      <- temporary file for body of INP file, CWI
C     BODYY2         26      <- temporary file for body of scatter file, CWI

C
C  OUTPUT FILES
C  ------------
C  Output are the following four files:
C
C  CP.INP,  CPY.INP, CPSCAT.DAT, CPYSCAT.DAT
C  or
C  LI.INP and LIY.INP, LISCAT.DAT, LIYSCAT.DAT
C
C  The .INP files are prepared for BOOT and RESIDUAL.
C  The .DAT files are prepared for SIGPLOT scatter plots.
C
C  The format of the Lillestroem files is different from
C  that of the Copenhagen files, because ZMIX is missing at
C  Lillestroem. Thus, where Copenhagen has columns for h/L and ZMIX,
C  Lillestroem has only a column with L.

C
C
C  REQUIRED SUBPROGRAMS:
C  ----------------------
C     One required subprogram is included with this code, namely
C
C     JULIAN   Function to determine Julian day number
C
C     Other required subprograms are found in the include file .
C        Read_ini_routines.for
C     These are used for retrieving information from the INI file.
C
C
C  USAGE:
C  ------
C     You must create a file, MODELLED.CPH (or MODELLED.LIL) containing 
C     modelled data (the format is described in the section 'Required files'; 
C     you may use a name of your choice for MODELLED.KIN). 
C
C     Before running the program, you must edit the INI file Combine_CphLil.ini
C     to suit your needs (using any text editor).
C
C     To run the program, doubleclick the executable.
C
C     Files with extension .INP and .DAT are produced, which can be used
C     for further processing by BOOT, RESIDUAL or SIGPLOT.
C     A 'Y' in the file name indicate that it refers to crosswind integrated
C     concentrations.
C
C
C    *****************  End of program header  ***************


      INTEGER QUAL, TMP_UNIT
      CHARACTER*12 ID_STR

      
      CHARACTER * 256 Results_path, Results_filename, Master_path,
     &   Results_file, MASTNAME, BTT_NAME, INP_FIL, SCAT_FIL,
     &   Loc, Prefix
      LOGICAL is_ini_found, is_found, getValueOnOff, cph
      external setIniFilename, getValue, getValueOnOff, downcase
     
      
      

C Check whether the option file 'Combine_CphLil.ini' exists.

      Call setIniFilename('Combine_CphLil.ini', is_ini_found)
      IF (is_ini_found) Then
         PRINT *, ' This program produces several output files.'
         PRINT *, ' They are for use by BOOT, RESIDUAL and SIGPLOT.'
         PRINT *, ' '
         WRITE (*,*)
     &      ' Settings from the file Combine_CphLil.ini are used.'
         PRINT *, ' '
      ELSE
         Goto 9010     
      End if         
      
      
      
C ----------------------------------------------------
C     Options from section [Files] in Combine_CphLil.ini
C ----------------------------------------------------

C     Retrieve parameters corresponding to certain keywords.

      call getValue('Files', 'Location',  Loc,
     &   '', is_found)
C     Location is either Cph or Lil.

      call downcase(Loc)
      If (Loc .eq. 'cph') then
         CPH = .TRUE.
         Print *, ' The location is Copenhagen.'
         PRINT *, ' '
      else if (Loc .eq. 'lil') then
         CPH = .FALSE.
         Print *, ' THe location is Lillestrom.'
         PRINT *, ' '
      else
         goto 9007
      end if
      


      call getValue('Files', 'Results_path',  Results_path,
     &   '', is_found)
      

      call getValue('Files', 'Results_file',  Results_filename,
     &   '', is_found)


      call getValue('Files', 'Master_path',  Master_path,
     &   '', is_found)

      Results_file = Trim(Results_path) // '\' // Trim(Results_filename)
      
      
      
      

C                                               ------------ Open files
C                                                                   ...

c     Open two input files to be merged and four temporary output files,
C     BODY1, BODY2, BODYY1 and BODYY2.
C     Later, two files containing header are opened and closed.
C     At the end of the process, the permanent output files are created
C     by a DOS copy command.


      


C     Open MODELLED.DAT
      OPEN(10, ERR=9001, FILE=Trim(Results_file),
     &        STATUS='OLD')
     

C     Open MASTER.CPH
         MASTNAME = Trim(Master_path) // '\' // 'MASTER.' // TRIM(Loc)

         OPEN(11,ERR=9003,FILE=MASTNAME, STATUS='OLD')



C        Open temporary files BODY1, BODY2, BODYY1, BODYY2.

         OPEN(12,FILE='BODY1',STATUS='UNKNOWN')
         OPEN(16,FILE='BODY2',STATUS='UNKNOWN')
         OPEN(22,FILE='BODYY1',STATUS='UNKNOWN')
         OPEN(26,FILE='BODYY2',STATUS='UNKNOWN')
         
         
C
C     Check whether the file with bottom lines exists:
      
         BTT_NAME = Trim(Master_path) // '\' // TRIM(Loc) // '.BTT'
         OPEN(13,ERR=9004,FILE= BTT_NAME,
     &      STATUS='OLD')
         CLOSE (13)


C     End of file opening statements      


C ---------------------------------------------------
      
    
     



C     ID string to be printed on scatter plot
      PRINT *, '      '
      PRINT *, 'Enter a short ID string (max 12 characters).'
      PRINT *, 'It is added to the title for a scatter plot ',
     &  'produced by SIGPLOT.'
      PRINT *, 'The title reads:'
      IF (CPH) PRINT *, 'Copenhagen scatter - <your string>'
      IF (.NOT.CPH) PRINT *, 'Lillestrom scatter - <your string>'
      PRINT *, '  ==>   '
      READ (5,705) ID_STR
705   FORMAT (A12)



C                                               -------- Merge two files
C                                               -------- (read and write)
C                                                                   ...


C     Observed values are in MASTER.CPH (unit 11, variable names
C     contain an 'N'). 
C     Modelled values are in MODELLED.CPH (unit 10, variable names
C     contain an 'M')
C     Skip headers in both files

      READ(10,*)
      READ(11,*)
      
      N_REC    = 0
      
C     NNOBS is required by BOOT version 2.0.
      NNOBS = 1
      
      
 
C     Read observed values (Unit 11, MASTER.CPH)
 9    If (loc .eq. 'cph') Then
          READ (11, *, END=9998, Err=9005) NYR, NMO, NDY, NHR,  DIST_N,
     &                C_OBS, CY_OBS, USTAR, H_L, ZMIX
       ELSE
C         (for Lillestrom, MASTER.LIL)
          READ (11, *, END=9998, Err=9005) NYR, NMO, NDY, NHR,  DIST_N,
     &                C_OBS, CY_OBS, USTAR, AL
      END if
 
C     Determine Julian date and a DATE-TIME parameter.
C     NHR is assumed to have 4 digits, e.g. 1015.
      JUL_N = JULIAN (NDY, NMO, NYR)
      D_TIME_N= 1000 * NYR + JUL_N + NHR/10000.

C LOOP
C     Read modelled values from MODELLED.CPH etc., one by one:

10    READ (10, *, END=9998, Err=9006) MYR, MMO, MDY, MHR,  DIST_M, 
     &   C_MOD, CY_MOD


C     Determine Julian date and a DATE-TIME parameter.
C     MHR is assumed to have 4 digits, e.g. 1015.
      JUL_M = JULIAN (MDY, MMO, MYR)
      D_TIME_M= 1000 * MYR + JUL_M + MHR/10000.

C     Read another modelled value if the dates etc. not yet match
      IF (D_TIME_M .LT. D_TIME_N) GO TO 10
      IF (D_TIME_M .GT. D_TIME_N) GO TO 900
      IF ( ABS(DIST_M - DIST_N) .GT. 0.0001 ) GO TO 10
     
C     At this point, date-time-distance values agree, so we can write
C     a line in the BODY files (BODY constitutes the main part of any of
C     the resulting files);


C     BODY1 has '1' as the first number in each record (NNOBS is 1).
     

C      Arc-wise maxima, body of INP:
          IF (cph) WRITE (12, 700) NNOBS,
     &      C_OBS, C_MOD, DIST_M, USTAR, H_L, ZMIX,
     &      MYR, MMO, MDY, MHR
          IF (.not.cph) WRITE (12, 701) NNOBS,
     &      C_OBS, C_MOD, DIST_M, USTAR, AL,
     &      MYR, MMO, MDY, MHR

C      Cross-wind integrated concentrations, body of INP:
          IF (cph) WRITE (22, 700) NNOBS,
     &      CY_OBS, CY_MOD, DIST_M, USTAR, H_L, ZMIX,
     &      MYR, MMO, MDY, MHR
          IF (.not.cph) WRITE (22, 701) NNOBS,
     &      CY_OBS, CY_MOD, DIST_M, USTAR, AL,
     &      MYR, MMO, MDY, MHR


C      Arc-wise maxima, body of scatter file:
          IF (cph) WRITE (16, 702)
     &      C_OBS, C_MOD, DIST_M, USTAR, H_L, ZMIX,
     &      MYR, MMO, MDY, MHR
          IF (.not.cph) WRITE (16, 703)
     &      C_OBS, C_MOD, DIST_M, USTAR, AL,
     &      MYR, MMO, MDY, MHR

C      Cross-wind integrated concentrations, body of scatter file:
          IF (cph) WRITE (26, 702)
     &      CY_OBS, CY_MOD, DIST_M, USTAR, H_L, ZMIX,
     &      MYR, MMO, MDY, MHR
          IF (.not.cph) WRITE (26, 703)
     &      CY_OBS, CY_MOD, DIST_M, USTAR, AL,
     &      MYR, MMO, MDY, MHR



          N_REC=N_REC+1

700       FORMAT (I1, 2F10.1, F7.2, F7.3, F7.1, F7.0, 3I3, I6)
701       FORMAT (I1, 2F13.1, F7.2, F7.3, F7.0, 3I3, I6)
702       FORMAT (2F10.1, F7.2, F7.3, F7.1, F7.0, 3I3, I6)
703       FORMAT (2F13.1, F7.2, F7.3, F7.0, 3I3, I6)

      GOTO 9


9998  CONTINUE

      CLOSE(12)
      CLOSE(16)
      CLOSE(22)   
      CLOSE(26)      


C     At this point, the temporary files BODY1, BODY2 etc. have been created.
C     Next, BODY is combined with headers and 'footer'.

C     Write Headers:                                  - Write headers
C                                                                ...

C     First, a header for the arc. max. .INP file is generated:

         OPEN(14,FILE='HDR.MAX',STATUS='UNKNOWN')
         IF (cph) WRITE (14, 901) N_REC, N_REC
         IF (.not.cph) WRITE (14, 911) N_REC, N_REC

901   format(1H , I4, '   2   1   4'/
     1     1H ,  I4,/,
     2 4H   ','C_OBS', 1H',' ' 3H  ','C_MOD',1H','   DIST',
     3'     u*    h/L  zmix  YR MO DY  HRE '/,
     4 2H ','Block of all data',1H' )

911   format(1H , I4, '   2   1   2'/
     1     1H ,  I4,/,
     2 7H      ','C_OBS', 1H',' ' 6H     ','C_MOD',1H','   DIST',
     3'     u*     L  YR MO DY  HRE '/,
     4 2H ','Block of all data',1H' )

      CLOSE(14)
      
      
      
C     Next, a header for the cross-w .INP file is generated:

         OPEN(14,FILE='HDR.CWI',STATUS='UNKNOWN')
         IF (cph) WRITE (14, 902) N_REC, N_REC
         IF (.not.cph) WRITE (14, 912) N_REC, N_REC

902   FORMAT(1H , I4, '   2   1   4'/
     1     1H ,  I4,/,
     2 4H   ','CY_OBS', 1H',' ' 3H  ','CY_MOD',1H',' DIST',
     3'     u*    h/L  zmix  YR MO DY  HRE '/,
     4 2H ','Block of all data',1H' )

912   format(1H , I4, '   2   1   4'/
     1     1H ,  I4,/,
     2 7H      ','CY_OBS', 1H',' ' 6H     ','CY_MOD',1H',' DIST',
     3'     u*     L  YR MO DY  HRE '/,
     4 2H ','Block of all data',1H' )

      CLOSE(14)


C     Then, headers for the .DAT files (scatter plot) are generated:
         OPEN(15,FILE='HDRDAT.MAX',STATUS='UNKNOWN')
         IF (CPH) WRITE (15, 903) ID_STR, N_REC
         IF (.NOT.CPH) WRITE (15, 913) ID_STR, N_REC

903   format(1H ,  'Copenhagen scatter  -  ',A12,/
     &     1H ,  'c\sub{obs}'/,
     &     1H ,  'c\sub{mod}'/,
     &     i4,' 1'/,
     &      ''' ''' )

913   format(1H ,  'Lillestrom scatter  -  ',A12,/
     &     1H ,  'c\sub{obs}'/,
     &     1H ,  'c\sub{mod}'/,
     &     i4,' 1'/,
     &      ''' ''' )

      CLOSE(15)

         OPEN(15,FILE='HDRDAT.CWI',STATUS='UNKNOWN')
         IF (CPH) WRITE (15, 904) ID_STR, N_REC
         IF (.NOT.CPH) WRITE (15, 914) ID_STR, N_REC

904   format(1H ,  'Copenhagen CWI scatter  -  ',A12,/
     &     1H ,  'cy\sub{obs}'/,
     &     1H ,  'cy\sub{mod}'/,
     &     i4,' 1'/,
     &      ''' ''' )

914   format(1H ,  'Lillestrom CWI scatter  -  ',A12,/
     &     1H ,  'cy\sub{obs}'/,
     &     1H ,  'cy\sub{mod}'/,
     &     i4,' 1'/,
     &      ''' ''' )

      CLOSE(15)


C                                               ------  Combine header,
C                                                        body & bottom
C                                                                   ...
C     4 output files are produced for either Copenhagen or Lillestrom
C     There is an .INP output file designed to be used by BOOT and
C     RESIDUAL, and an .DAT file designed to be used by SIGPLOT for
C     scatter plots. This goes for both arc-wise maxima and crosswind
C     integrated concentrations.

      PREFIX = TRIM(Results_path) // '\'

      IF (CPH) THEN
C     COPENHAGEN
      Print *, 'copy HDR.MAX+BODY1+'//Trim(BTT_NAME)//
     &      ' ' // TRIM(Prefix) // 'CP.INP'
      PRINT *, ' '
         CALL SYSTEM
     &      ('COPY  HDR.MAX+BODY1+'//Trim(BTT_NAME)//
     &      ' ' // TRIM(Prefix) // 'CP.INP')

         CALL SYSTEM
     &      ('COPY  HDR.CWI+BODYY1+'//Trim(BTT_NAME)//
     &      ' ' // TRIM(Prefix) // 'CPY.INP')

         CALL SYSTEM
     &      ('COPY  HDRDAT.MAX+BODY2 ' // TRIM(Prefix) // 'CPSCAT.DAT')

         CALL SYSTEM
     &      ('COPY  HDRDAT.CWI+BODYY2 ' // TRIM(Prefix) //'CPYSCAT.DAT')
     

         PRINT  *, ' '
         PRINT  *, ' The following files have been created in folder'
         Print  * , Trim(Results_path), ':'
         PRINT  *, '                                   CP.INP     '
         PRINT  *, '                                   CPY.INP    '
         PRINT  *, '                                   CPSCAT.DAT '
         PRINT  *, '                                   CPYSCAT.DAT'
         PRINT  *, '  '
         PRINT  *, ' The files with extension .INP can be used as ',
     &     'input for BOOT and RESIDUAL.'
         PRINT  *, ' The files with extension .DAT can be used as ',
     &     'input for SIGPLOT.'
         PRINT  *, ' '


      else if (.NOT.CPH) then
C     LILLESTROM
         CALL SYSTEM
     &      ('COPY  HDR.MAX+BODY1+'//Trim(BTT_NAME)//
     &      ' ' // TRIM(Prefix) // 'LI.INP')

         CALL SYSTEM
     &      ('COPY  HDR.CWI+BODYY1+'//Trim(BTT_NAME)//
     &      ' ' // TRIM(Prefix) // 'LIY.INP')


         CALL SYSTEM
     &      ('COPY  HDRDAT.MAX+BODY2 ' // TRIM(Prefix) // 'LISCAT.DAT')
     

         CALL SYSTEM
     &      ('COPY  HDRDAT.CWI+BODYY2 ' // TRIM(Prefix) //'LIYSCAT.DAT')


         PRINT  *, ' '
         PRINT  *, ' The following files have been created in folder'
         Print  * , Trim(Results_path), ':'
         PRINT  *, '                                   LI.INP     '
         PRINT  *, '                                   LIY.INP    '
         PRINT  *, '                                   LISCAT.DAT '
         PRINT  *, '                                   LIYSCAT.DAT'
         PRINT  *, ' '
         PRINT  *, '  '
         PRINT  *, ' The files with extension .INP can be used as ',
     &     'input for BOOT and RESIDUAL.'
         PRINT  *, ' The files with extension .DAT can be used as ',
     &     'input for SIGPLOT.'
         PRINT  *, ' '


      ENDIF


      PRINT  *
      PRINT *, 'Number of data records: ', N_REC
      PRINT  *

C     Cleaning up:
      CALL SYSTEM
     &   ('DEL BODY1')
      CALL SYSTEM
     &   ('DEL BODY2')
      CALL SYSTEM
     &   ('DEL BODYY1')
      CALL SYSTEM
     &   ('DEL BODYY2')
         CALL SYSTEM
     &      ('DEL HDRDAT.MAX')
         CALL SYSTEM
     &      ('DEL HDRDAT.CWI')
         CALL SYSTEM
     &      ('DEL HDR.MAX')
         CALL SYSTEM
     &      ('DEL HDR.CWI')


      GOTO 9999


C   Error messages:

900   PRINT *,' Error: mismatch between sets of Date-Time-Distance '
      PRINT *,' values in ' // Trim(MASTNAME) // ' detected.'
      PRINT *,' Execution stopped.'
      PRINT *, ' '
      PRINT *, 'Date etc. from ' // Trim(MASTNAME) // ': '
      PRINT *, NYR, NMO, NDY, NHR, DIST_N
      PRINT *, ' '
      PRINT *, 'Date etc. from ' // Trim(Results_file) // ':'
      PRINT *,     MYR, MMO, MDY, MHR, DIST_M
      GOTO 9999

     
C   Error messages if certain files cannot be opened or read:
C


 9001 PRINT *,' File ' // Trim(Results_file) // ' cannot be opened.'
      PRINT *,' It is expected to contain modelled results.'
      PRINT *,
     & ' Edit the file Combine_CphLil.ini to specify where it is ',
     & 'located.'
      
      PRINT *,' Execution stopped.'
      PRINT *, ' '
      GOTO 9999




 9003 PRINT *,' '
      PRINT *,' Error opening file ', MASTNAME
      PRINT *,' This file is supplied with the Model Validation Kit.'
      PRINT *,
     & ' Edit the file Combine_CphLil.ini to specify where it is',
     & ' located.'
      PRINT *,' '
      GOTO 9999




 9004 PRINT *,' '
      PRINT *,' Error opening file ' // Trim(Master_path) // '\' // 
     &   TRIM(Loc) // '.BTT'
      PRINT *,' This file is supplied with the Model Validation Kit.'
      PRINT *,
     & ' Edit the file Combine_CphLil.ini to specify where it is ',
     & 'located.'
      PRINT *,' '
      GOTO 9999


 9005 PRINT *,' '
      PRINT *,' Error while reading file '// MASTNAME
      PRINT *,' This file is supplied with the Model Validation Kit.'
      PRINT *,
     &   ' Check that it contains what it is supposed to.'
      PRINT *,' '
      GOTO 9999


 9006 PRINT *,' '
      PRINT *,' Error while reading the file '// Trim(Results_file)
      PRINT *,' This is a file containing model results.'
      PRINT *,
     &   ' Check that it conforms with the format specifications.'
      PRINT *,' '
      GOTO 9999



 9007 PRINT *,' '
      PRINT *,' Error: The location should be either Copenhagen or ',
     &   'Lillestrom'      
      PRINT *,
     & ' Edit the file Combine_CphLil.ini, so Location is either ',
     &   'CPH or LIL.'
      PRINT *,' '
      GOTO 9999



      
9010  Print *,
     &   'A required file  -  Combine_CphLil.ini - cannot be found.'
      Print *, 'Combine_CphLil.ini is supposed to contain information',
     & ' on file locations.'
      Print *, 'Program stopped.       '
      Goto 9999






9999  STOP
      END

C
C             ***************************************
C              E N D   O F   M A I N   P R O G R A M
C             ***************************************
C



      Function  JULIAN (DAY, MONTH, YEAR)
C
C     The Julian day is determined. The first formula would be exact if
C     all 12 months of the year had 31 days. The second formula corrects
C     for the different lengths of months, the third for leap years.

      INTEGER DAY, YEAR

      JULDAY = 31*(MONTH-1) + DAY
      IF (MONTH.GT.2)  JULDAY = JULDAY - INT( 0.4*MONTH+2.3 )
      IF (MONTH.GT.2 .AND. YEAR/4*4 .EQ. YEAR)  JULDAY = JULDAY+1

      JULIAN = JULDAY

      RETURN
      END



      Include 'Read_ini_routines.for'
