C     Last change:  HO    2 Oct 2005    6:17 pm
      Program Combine_Indi
C
C                   *************************
C                     C O M B I N E _ I N D I       (960205, revised Sept 2005.
C                   *************************
C
C
C     National Environmental Research Institute
C     Department of Atmospheric Environment
C     P.O. Box 358
C     DK-4000  Roskilde
C     DENMARK
C
C     The program was written by:
C      Helge Rordam Olesen
C
C      Phone:  +45 46 30 12 00 (ext. 1151)
C      Fax:    +45 46 30 12 14
C      E-mail: hro@dmu.dk
C
C
C
C  DESCRIPTION:
C  ------------
C     Utility for selecting data lines of interest in a file with
C     modelled Indianapolis data, and combining them with observed
C     tracer concentration data.
C
C     The two files to be combined are
C     (1) modelled Indianapolis data
C     (2) a master file with information on observed tracer concentrations.
C   
C     The outcome of the efforts are files that can be used as input for
C     the BOOT, RESIDUAL and SIGPLOT programmes (or for other purposes).
C
C
C  PROGRAM LOGIC:
C  ---------------
C     Various options (filenames and pathnames) are specified in the INI
C     file Combine_Indi.ini, which must be present in the same folder as 
C     the executable.
C
C     First of all, this INI file is read.
C
C     The input files MODELLED.IND (user-created) and MASTER.IND
C     (supplied) are opened.  
C
C     The user is asked to provide information on quality level, and
C     give a descriptive label to be used for plots.
C
C     Records from the two files are read. Records from MASTER.IND
C     with a quality level lower than specified by the user are
C     immediately skipped, and another record is read.
C     As soon as a record is read (from either file), it is checked
C     whether it corresponds in date, time and distance to the current
C     record from the other file.
C
C     Each time the data correspond in date, time and distance, while
C     the criteria on quality level are fulfilled, a record is written
C     to two almost identical temporary files, BODY1 and BODY2.
C
C     When the loops end, the BODY files contain essentially all information to
C     be output. However, headers must be included, and in the case of
C     the file of type .INP, also a 'footer' must be added.
C
C     Thus, two files with headers are created.
C
C     Finally, a DOS copy command is used in order to combine headers,
C     BODY and (in case of .INP) footer to the final output files.
C
C  DATA DICTIONARY:
C  -----------------
C     Mainly, usage of the variables is apparent from the program code.
C     Note, however:
C
C     Date and time variables from MODELLED.IND are designated by a name
C     with an 'M';
C     MDY, MMO, MYR
C     MHR
C     JUL_M
C     D_TIME_M
C
C     while the corresponding variables from MASTER.IND are designated
C     by a name with an 'N':
C
C     NDY, NMO, NYR
C     NHR
C     JUL_N
C     D_TIME_N
C
C  REQUIRED FILES:
C  ----------------
C  Input:
C
C  Four input files are required:
C     Combine_Indi.INI
C     MODELLED.IND  (the file can have any name)
C     MASTER.IND
C     IND.BTT
C
C  Combine_Indi.INI contains user specifications of filenames and path names.
C
C  MODELLED.IND (unit 10) is the primary input file. 
C     Although the name MODELLED.IND is used here, the file can
C     have any name. It should be prepared by the user and
C     contain modelled concentrations.
C     It must be placed in a folder specified in Combine_Indi.ini.
C     Its name is also specified in Combine_Indi.ini.
C
C     The format of MODELLED.IND should be the following:
C     -  There is a one-line heading
C     -  There should be six columns (or more), separated by blanks
C        (the values are read using free-format input).
C     -  Typically, there is a line for all 2040 arc-hours (170
C        hours times 12 distances). However, it is acceptable that
C        only SOME of these 2040 arc-hours are represented in the file.
C     An example of the first thirteen lines of a complete MODELLED.IND
C     file is shown below.
C     There are twelve possible values for the distance to arcs, DIST.
C     These are indicated in the example below. It is important that the
C     distance values in the file are exactly as prescribed (e.g.
C     12.000 and not 11.995).
C
C     YR  MO DY   HR   DIST  ARCMAX/Q 
C     85  9  16   11   0.250     33.
C     85  9  16   11   0.500    479.
C     85  9  16   11   0.750    643.
C     85  9  16   11   1.000    587.
C     85  9  16   11   1.500    403.
C     85  9  16   11   2.000    337.
C     85  9  16   11   3.000    254.
C     85  9  16   11   4.000    200.
C     85  9  16   11   6.000    141.
C     85  9  16   11   8.000    109.
C     85  9  16   11  10.000     90.
C     85  9  16   11  12.000     76.

C
C     MASTER.IND (unit 11) is provided together with this programme.
C     It has a one-line heading, followed by 1511 data lines.
C
C     IND.BTT (unit 13) is provided together with this programme. It is
C     automatically copied into the bottom part of the output file of type .INP.
C
C     These two files must be placed in a folder specified in Combine_Indi.ini .
C
C     Below follows a list of input files and temporary files used
C     during program execution:
C
C     File name     Unit
C     MODELLED.IND   10      <- input prepared by user (the name can be 
C                                specified in Combine_Indi.ini).
C     MASTER.IND     11      <- supplied with this code
C     BODY1          12      <- temporary file
C     BODY2          16      <- temporary file
C     IND.BTT        13      <- supplied with this code
C                               (bottom part of .INP files).
C     IN_HDR.INP     14      <- temporary file, header for .INP files
C     IN_HDR.DAT     15      <- temporary file, header for .DAT files

C
C     Concerning output files, see the next section.
C
C  OUTPUT FILES
C  ------------
C     Output consists of two files:
C
C        one of type .INP (designed as input to BOOT and RESIDUAL)
C     and
C        one of type .DAT (designed for SIGPLOT scatter plot).
C
C     The naming conventions are as follows:
C
C     The file names depend on the user-prescribed quality level,
C     LOWLEV.  For instance, if the user chooses 2 as quality level,
C     (meaning that data with a quality index of 2 and 3 are included)
C     LOWLEV is 2.  The digit in the file name is identical to LOWLEV.
C
C     Thus, IN2.INP concerns INdianapolis data, with QUAL having values
C     from 2-3.
C
C     The following file names are possible:
C
C      Files for                     Files for
C      BOOT and RESIDUAL            scatter plots
C         IN0.INP                     INSCAT0.DAT
C         IN1.INP                     INSCAT1.DAT
C         IN2.INP                     INSCAT2.DAT
C         IN3.INP                     INSCAT3.DAT
C
C
C  REQUIRED SUBPROGRAMS:
C  ----------------------
C     One required subprogram is included with this code, namely
C
C     JULIAN   Function to determine Julian day number
C
C     Other required subprograms are found in the include file .
C        Read_ini_routines.for
C     These are used for retrieving information from the INI file.
C
C
C  USAGE:
C  ------
C     You must create a file, MODELLED.IND, containing modelled data (the
C     format is described in the section 'Required files'; you may use a
C     name of your choice for MODELLED.IND). 
C
C     Before running the program, you must edit the INI file Combine_Indi.ini
C     to suit your needs (using any text editor).
C
C     To run the program, doubleclick the executable.
C
C     You are asked which quality levels you want to consider.
C     Typically, you should choose 3 (only data with quality index QUAL
C     equal to 3 are included in the output) or 2 (data with QUAL equal
C     to 2 or 3 are included).
C
C     Files with extension .INP and .DAT are produced, which can be used
C     for further processing by BOOT, RESIDUAL or SIGPLOT.
C
C
C    *****************  End of program header  ***************


      INTEGER QUAL, TMP_UNIT
      CHARACTER*12 ID_STR

      
      CHARACTER * 256 Results_path, Results_filename, Master_path,
     &   Results_file, MASTNAME, BTT_NAME, INP_FIL, SCAT_FIL
      LOGICAL is_ini_found, is_found, getValueOnOff     
      external setIniFilename, getValue, getValueOnOff
     
      
      

C Check whether the option file 'Combine_Indi.ini' exists.

      Call setIniFilename('Combine_Indi.ini', is_ini_found)
      IF (is_ini_found) Then
         PRINT *, ' This program produces several output files.'
         PRINT *, ' They are for use by BOOT, RESIDUAL and SIGPLOT.'
         PRINT *, ' '
         WRITE (*,*)
     &      ' Settings from the file Combine_Indi.ini are used.'
         PRINT *, ' '
      ELSE
         Goto 9010     
      End if         
      
      
      
C ----------------------------------------------------
C     Options from section [Files] in Combine_Indi.ini
C ----------------------------------------------------

C     Retrieve parameters corresponding to certain keywords.

      call getValue('Files', 'Results_path',  Results_path,
     &   '', is_found)


      call getValue('Files', 'Results_file',  Results_filename,
     &   '', is_found)


      call getValue('Files', 'Master_path',  Master_path,
     &   '', is_found)

      Results_file = Trim(Results_path) // '\' // Trim(Results_filename)
      
      
      
      

C                                               ------------ Open files
C                                                                   ...

c     Open two input files to be merged and two temporary output files,
C     BODY1 and BODY2 .
C     Later, two files containing header are opened and closed.
C     At the end of the process, the permanent output files are created
C     by a DOS copy command.


      


C     Open MODELLED.DAT
      OPEN(10, ERR=9001, FILE=Trim(Results_file),
     &        STATUS='OLD')
     

C     Open MASTER.IND
         MASTNAME = Trim(Master_path) // '\' // 'MASTER.IND'

         OPEN(11,ERR=9003,FILE=MASTNAME, STATUS='OLD')



C        Open temporary files BODY1 and BODY2

         OPEN(12,FILE='BODY1',STATUS='UNKNOWN')
         OPEN(16,FILE='BODY2',STATUS='UNKNOWN')                  
         
         
C
C     Check whether the file with bottom lines exists:
      
         BTT_NAME = Trim(Master_path) // '\IND.BTT'
         OPEN(13,ERR=9004,FILE=BTT_NAME,
     &      STATUS='OLD')
         CLOSE (13)


C     End of file opening statements      


C ---------------------------------------------------
      
      
      

C     Which output does the user wish?
      PRINT *, 'Choose quality level of data: '
      PRINT *, '      0:   QUAL can have any value '
      PRINT *, '      1:   QUAL 1, 2 and 3    '
      PRINT *, '      2:   QUAL 2 and 3    '
      PRINT *, '      3:   only QUAL 3     '
      PRINT *, '      '
      PRINT *, '                  ......(2 or 3 is recommended)...'
      PRINT *, '      '
      READ (5,*) LOWLEV
      IF (LOWLEV .LT. 0 .OR. LOWLEV .GT.3 ) GO TO 9008

C     ID string to be printed on scatter plot
      PRINT *, '      '
      PRINT *, 'Enter a short ID string (max 12 characters).'
      PRINT *, 'It is added to the title for a scatter plot ',
     &  'produced by SIGPLOT.'
      PRINT *, 'The title reads:'
      PRINT *, 'Indianapolis scatter - <your string>'
      PRINT *, '  ==>   '
      READ (5,705) ID_STR
705   FORMAT (A12)



C                                               -------- Merge two files
C                                               -------- (read and write)
C                                                                   ...


C     Observed values are in MASTER.IND (unit 11, variable names
C     contain an 'N'). 
C     Modelled values are in MODELLED.IND (unit 10, variable names
C     contain an 'M')
C     Skip headers in both files

      READ(10,*)
      READ(11,*)
      
      N_REC    = 0
      D_TIME_N = 0.
      D_TIME_M = 0.
      
C     NNOBS is required by BOOT version 2.0.
      NNOBS = 1

     
      
C     Observed values (MASTER.IND)
 9    READ (11, *, END=9998, Err=9005) NYR, NMO, NDY, NHR,  DIST_N,
     &                C_OBS, QUAL, USTAR, H_L, ZMIX
C     Take next record if QUAL is less that the user-specified value.
      IF (QUAL .LT. LOWLEV) GO TO 9

C     Determine Julian date and a DATE-TIME parameter.
C     This only works properly if NHR < 100 (hour variable).
      JUL_N = JULIAN (NDY, NMO, NYR)
      D_TIME_N = 1000 * NYR + JUL_N + NHR/100.

C     Go to comparison of sort keys
      GOTO 12


10    READ (10, *, END=9998, Err=9006) MYR, MMO, MDY, MHR,  DIST_M, 
     &   C_MOD

C     Determine Julian date and a DATE-TIME parameter.
C     This only works properly if MHR < 100 (hour variable).
      JUL_M = JULIAN (MDY, MMO, MYR)
      D_TIME_M= 1000 * MYR + JUL_M + MHR/100.
      GOTO 12

C     Now compare N-variables to M-variables.  Two sort keys are used:
C     a date-time values such as 85259.23 (indicating year 85, Julian
C     date 259, 23 hours.) and a distance value.
C     If the N-variable is smallest, read another N-record (statement
C     9). Conversely, if the M-variable is smallest, read next M-record
C     (statement 10).
C     We arrive at this statement (label 12) after every read.

12    IF (D_TIME_N .LT.  D_TIME_M ) GO TO 9
      IF (D_TIME_M .LT.  D_TIME_N ) GO TO 10

C     If we are here, date-time values agree. Records are compared
C     according to the secondary sort key, distance:

      IF (DIST_N .LT.  DIST_M ) GO TO 9
      IF (DIST_M .LT.  DIST_N ) GO TO 10

C     Final check, ought not be necessary:
      IF ( (D_TIME_N .NE.  D_TIME_M ) .OR.
     &   ( ABS(DIST_M - DIST_N) .GT. 0.0001 )  ) GOTO 900

C     At this point, date-time-distance values agree, so we can write
C     a line in the BODY files (BODY constitutes the main part of any of the
C     resulting files);


C     BODY1 has '1' as the first number in each record (NNOBS is 1).

             WRITE (12, 700) NNOBS,
     &         C_OBS, C_MOD, DIST_M, USTAR, MHR, ZMIX,
     &         MYR, MMO, MDY, MHR, QUAL
             
700          FORMAT (I1, 2F7.1, F7.2, F7.3, I7, F7.0, 5I3)


             WRITE (16, 702)
     &         C_OBS, C_MOD, DIST_M, USTAR, MHR, ZMIX,
     &         MYR, MMO, MDY, MHR, QUAL
             
702          FORMAT (2F7.1, F7.2, F7.3, I7, F7.0, 5I3)

            N_REC=N_REC+1

      GO TO  9

900   PRINT *,' Error: mismatch between sets of Date-Time-Distance '
      PRINT *,' values in MASTER.IND and MODELLED.IND detected.'
      PRINT *,' Execution stopped.'
      PRINT *, 'Date etc. from MASTER.IND: ', NYR, NMO, NDY, NHR, DIST_N
      PRINT *, 'Date etc. from ' // Trim(Results_file), 
     &      MYR, MMO, MDY, MHR, DIST_M
      GOTO 9999

9998  CONTINUE

      CLOSE(12)
      CLOSE(16)      


C     At this point, the temporary files BODY1 and BODY2 have been created.
C     Next, BODY is combined with headers and 'footer'.

C     Write Headers:                                  - Write headers
C                                                                ...
C     First, a header for the .INP file is generated:

         OPEN(14,FILE='IN_HDR.INP',STATUS='UNKNOWN')
         write (14, 901) N_REC, N_REC

901   format(1H , I4, '   2   1   4'/
     1     1H ,  I4,/,
     2 2H ','C_OBS', 1H',' ' 1H','C_MOD',1H',' DIST     u*     HR  ',
     3'zmix  YR MO DY HR QUAL'/,
     4 2H ','Block of all data',1H' )

      CLOSE(14)

C     Next, a header for the .DAT file (scatter plot) is generated:
         OPEN(15,FILE='IN_HDR.DAT',STATUS='UNKNOWN')
         write (15, 902) ID_STR, N_REC

902   format(1H ,  'Indianapolis scatter  -  ',A12,/
     &     1H ,  'c\sub{obs}'/,
     &     1H ,  'c\sub{mod}'/,
     &     i4,' 1'/,
     &      ''' ''' )


      CLOSE(15)

C                                               ------  Combine header,
C                                                        body & bottom
C                                                                   ...

C     The names of the resulting output files depend on LOWLEV
C     There is an .INP output file designed to be used by BOOT and
C     RESIDUAL, and an .DAT file designed to be used by SIGPLOT for
C     scatter plots.  The digit in the file name is identical to LOWLEV.
C     E.g., IN1_.INP concerns INdianapolis data, with QUAL having values
C     from 1-3.
C

      IF (LOWLEV .EQ. 0) THEN
         INP_FIL = 'IN0.INP'
         SCAT_FIL = 'IN0.DAT'

      ELSE IF (LOWLEV .EQ. 1) THEN
         INP_FIL = 'IN1.INP'
         SCAT_FIL = 'INSCAT1.DAT'

      ELSE IF (LOWLEV .EQ. 2) THEN
         INP_FIL = 'IN2.INP'
         SCAT_FIL = 'INSCAT2.DAT'

      ELSE IF (LOWLEV .EQ. 3) THEN
         INP_FIL = 'IN3.INP'
         SCAT_FIL = 'INSCAT3.DAT'
      ENDIF
      
      
      INP_FIL=Trim(Results_path) // '\' // Trim(inp_fil)
      SCAT_FIL=Trim(Results_path) // '\' // Trim(SCAT_fil)

C        Copy header, data (file BODY1) and bottom lines (in file
C        with name BTT_NAME) to an .INP file
          CALL SYSTEM
     &   ('COPY  IN_HDR.INP+ BODY1 + '//Trim(BTT_NAME)//' '//INP_FIL)

         CALL SYSTEM
     &      ('COPY  IN_HDR.DAT+ BODY2 '//SCAT_FIL)

         PRINT  *
         PRINT  *
         PRINT  *, ' The file ', TRIM(INP_FIL), ' has been created.'
         PRINT  *, ' It can be used as input for BOOT and RESIDUAL.'
         PRINT  *
         PRINT  *
         PRINT  *

         PRINT  *, ' The file ', TRIM(SCAT_FIL), ' has been created.'
         PRINT  *, ' It can be used as input for SIGPLOT.'
         PRINT  *





      PRINT  *
      PRINT *, 'Number of data records: ', N_REC
      PRINT  *

C     Cleaning up:
      CALL SYSTEM
     &   ('DEL BODY1')
      CALL SYSTEM
     &   ('DEL BODY2')
         CALL SYSTEM
     &      ('DEL IN_HDR.INP')
         CALL SYSTEM
     &      ('DEL IN_HDR.DAT')

      GOTO 9999


9008  PRINT *,' This value is not a valid quality level!'

      GOTO 9999
      
      


C   Error messages if certain files cannot be opened or read:
C


 9001 PRINT *,' File ' // Trim(Results_file) // ' cannot be opened.'
      PRINT *,' It is expected to contain modelled results.'
      PRINT *,
     & ' Edit the file Combine_Indi.INI to specify where it is located.'
      
      PRINT *,' Execution stopped.'
      PRINT *, ' '
      GOTO 9999




 9003 PRINT *,' '
      PRINT *,' Error opening file ', MASTNAME
      PRINT *,' This file is supplied with the Model Validation Kit.'
      PRINT *,
     & ' Edit the file Combine_Indi.INI to specify where it is located.'
      PRINT *,' '
      GOTO 9999




 9004 PRINT *,' '
      PRINT *,' Error opening file ' // Trim(Master_path) // '\IND.BTT'
      PRINT *,' This file is supplied with the Model Validation Kit.'
      PRINT *,
     & ' Edit the file Combine_Indi.INI to specify where it is located.'
      PRINT *,' '
      GOTO 9999


 9005 PRINT *,' '
      PRINT *,' Error while reading file '// MASTNAME
      PRINT *,' This file is supplied with the Model Validation Kit.'
      PRINT *,
     &   ' Check that it contains what it is supposed to.'
      PRINT *,' '
      GOTO 9999


 9006 PRINT *,' '
      PRINT *,' Error while reading the file '// Trim(Results_file)
      PRINT *,' This is a file containing model results.'
      PRINT *,
     &   ' Check that it conforms with the format specifications.'
      PRINT *,' '
      GOTO 9999


      
9010  Print *, 'A required file  -  Combine_Indi.ini - cannot be found.'
      Print *, 'Combine_Indi.ini is supposed to contain information ',
     & ' on file locations.'
      Print *, 'Program stopped.       '
      Goto 9999






9999  STOP
      END

C
C             ***************************************
C              E N D   O F   M A I N   P R O G R A M
C             ***************************************
C



      Function  JULIAN (DAY, MONTH, YEAR)
C
C     The Julian day is determined. The first formula would be exact if
C     all 12 months of the year had 31 days. The second formula corrects
C     for the different lengths of months, the third for leap years.

      INTEGER DAY, YEAR

      JULDAY = 31*(MONTH-1) + DAY
      IF (MONTH.GT.2)  JULDAY = JULDAY - INT( 0.4*MONTH+2.3 )
      IF (MONTH.GT.2 .AND. YEAR/4*4 .EQ. YEAR)  JULDAY = JULDAY+1

      JULIAN = JULDAY

      RETURN
      END



      Include 'Read_ini_routines.for'
